/**
 *    ListSetting.java
 *    Copyright (C) 2010 New Zealand Digital Library, http://expeditee.org
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.expeditee.setting;

import java.util.LinkedList;
import java.util.List;

import org.expeditee.core.Representable;
import org.expeditee.gui.Frame;
import org.expeditee.gui.FrameIO;
import org.expeditee.items.Text;
import org.expeditee.settings.UserSettings;

public abstract class ListSetting<T> extends VariableSetting {

	protected List<T> _default;
	protected List<T> _value;
	
	public ListSetting(String tooltip, String name, List<T> value) {
		super(tooltip, name);
		_default = new LinkedList<T>(value);
		_value = value;
	}
	
	public ListSetting(String tooltip, String name) {
		this(tooltip, name, new LinkedList<T>());
	}
	
	public List<T> get() {
		return _value;
	}
	
	public void set(List<T> value) {
		_value = value;
	}
	
	public abstract boolean setSetting(Text text);
	
	public void setDefault(List<T> value) {
		_default = new LinkedList<T>(value);
	}
	
	public void reset() {
		_value = new LinkedList<T>(_default);
	}

	@Override
	public Text generateRepresentation(String label, String frameset) {
		Text t = new Text(label);
		
		if (!_value.isEmpty()) {
			Frame listFrame = FrameIO.CreateFrame(frameset, null, null);
			listFrame.getTitleItem().delete();
			
			int x = UserSettings.TitlePosition.get();
			int y = UserSettings.TitlePosition.get();
			for (T v: _value) {
				if (v == null) continue;
				Text representation;
				if (v instanceof Representable) {
					representation = ((Representable) v).generateRepresentation("List Entry", "");
					representation.setPosition(x, y);
					representation.setID(listFrame.getNextItemID());
					listFrame.addItem(representation);
				} else {
					representation = listFrame.addText(x, y, v.toString(), null);
				}
				y += representation.getBoundsHeight();
			}
			FrameIO.ForceSaveFrame(listFrame);
			t.setLink(listFrame.getName());
		}
		
		return t;
	}
}
